Module: GraphQL::Stitching::Executor::PathAccess
- Included in:
- RootSource, TypeResolverSource
- Defined in:
- lib/graphql/stitching/executor/path_access.rb
Overview
Utilities for traversing aggregate executor data along planned paths.
Utilities for traversing aggregate executor data along planned paths.